@@ -1,8 +1,9 @@
 `plot.plspm.groups` <-
 function(x,...)
 {
+    test <- ifelse (x$settings[[3]]=="bootstrap", "Bootstrap test", "Permutation test")
     par(mar=c(8, 4, 4, 2))
-    barplot(t(x$test[,2:3]), main="Path coefficients between groups", cex.main=1, 
+    barplot(t(x$test[,2:3]), main=c("Path coefficients between groups",test), cex.main=1, 
             ylim=c(min(x$test[,2:3]),1.20*max(x$test[,2:3])),
             beside=T, border=NA, col=c("skyblue","blue4"), names.arg=rep("",nrow(x$test)))
     abline(h=0)
