@@ -2,12 +2,13 @@
 function(x, ...)
 {
     Name <- c("$unidim", "$outer.mod", "$inner.mod", "$latents", "$scores", "$out.weights",  
-             "$loadings", "$path.coefs", "$r.sqr", "$outer.cor", "$inner.sum", "$gof", "$effects")
+             "$loadings", "$path.coefs", "$r.sqr", "$outer.cor", "$inner.sum", "$gof", 
+             "$effects", "$data")
     Description <- c("unidimensionality", "outer model", "inner model", "scaled LVs (variance=1)", 
        "re-scaled LVs for scaled=FALSE", "outer weights", "measurement loadings", 
        "path coefficients", "R-squared", "outer correlations", "summary inner model", 
-       "goodnes-of-fit", "total effects")
-    if (length(x)==15) {
+       "goodnes-of-fit", "total effects", "original data matrix")
+    if (length(x)==16) {
        Name <- c(Name, "$boot", "$boot$weights", "$boot$loadings", "$boot$paths",
                  "$boot$efects", "$boot$rsq")
        Description <- c(Description, "bootstrap results", "bootstrap weights", 
